import os
import unittest
from __main__ import vtk, qt, ctk, slicer

class CardiacMRIToolkitSelfTest:
  def __init__(self, parent):
    parent.title = "CardiacMRIToolkitSelfTest" 
    parent.categories = ["Testing.TestCases"]
    parent.dependencies = []
    parent.contributors = ["Salma Bengali (CARMA), Alan Morris (CARMA), Brian Zenger (CARMA), Josh Cates (CARMA), Rob MacLeod (CARMA)"]
    parent.helpText = """
    This module was developed as a self test for the Namic Slicer Tutorial Contest 2013    """
    parent.acknowledgementText = """
""" # replace with organization, grant and thanks.
    self.parent = parent

    # Add this test to the SelfTest module's list for discovery when the module
    # is created.  Since this module may be discovered before SelfTests itself,
    # create the list if it doesn't already exist.
    try:
      slicer.selfTests
    except AttributeError:
      slicer.selfTests = {}
    slicer.selfTests['CardiacMRIToolkitSelfTest'] = self.runTest

  def runTest(self):
    tester = CardiacMRIToolkitSelfTestCase()
    tester.runTest()
    
#
# CardiacMRIToolkitSelfTestWidget
#

class CardiacMRIToolkitSelfTestWidget:
  def __init__(self, parent = None):
   if not parent:
     self.parent = slicer.qMRMLWidget()
     self.parent.setLayout(qt.QVBoxLayout())
     self.parent.setMRMLScene(slicer.mrmlScene)
   else:
     self.parent = parent
   self.layout = self.parent.layout()
   if not parent:
     self.setup()
     self.parent.show()
     
  def setup(self):
   #Instantiate and connect widgets ... 
   
   # reload button
   # (use this during development, but remove it when delivering
   #  your module to users)
   self.reloadButton = qt.QPushButton("Reload")
   self.reloadButton.toolTip = "Reload this module."
   self.reloadButton.name = "CardiacMRIToolkitSelfTest Reload"
   self.layout.addWidget(self.reloadButton)
   self.reloadButton.connect('clicked()', self.onReload)

   # Collapsible button
   testsCollapsibleButton = ctk.ctkCollapsibleButton()
   testsCollapsibleButton.text = "A collapsible button"
   self.layout.addWidget(testsCollapsibleButton)

   # Layout within the collapsible button
   formLayout = qt.QFormLayout(testsCollapsibleButton)
   
   # test buttons
   tests = ( ("Part 1: DownloadMRIData",self.onDownloadMRIData),("Part 2: DownloadLAEndoSegmentation", self.onLAEndoSegmentation), ("Part 3: AxialDilate", self.onAxialDilate),("Part 4: BooleanRemove", self.onBooleanRemove), ("Part 5: DownloadLAWallCleanup", self.onDownloadLAWallCleanup), ("Part 6: PVAntrumCut", self.onPVAntrumCut), ("Part 7: EndoNVIsosurface", self.onEndoNVIsosurface), ("Part 8: AutomaticLeftAtrialScarAndIsosurface", self.onAutomaticLeftAtrialScar) )

   for text,slot in tests:
      testButton = qt.QPushButton(text)
      testButton.toolTip = "Run the test."
      formLayout.addWidget(testButton)
      testButton.connect('clicked(bool)', slot)

   # Add vertical spacer
   self.layout.addStretch(1)
    
  def onReload(self,moduleName="CardiacMRIToolkitSelfTest"):
    """Generic reload method for any scripted module.
    ModuleWizard will subsitute correct default moduleName.
    """
    import imp, sys, os, slicer

    widgetName = moduleName + "Widget"

    # reload the source code
    # - set source file path
    # - load the module to the global space
    filePath = eval('slicer.modules.%s.path' % moduleName.lower())
    p = os.path.dirname(filePath)
    if not sys.path.__contains__(p):
      sys.path.insert(0,p)
    fp = open(filePath, "r")
    globals()[moduleName] = imp.load_module(
        moduleName, fp, filePath, ('.py', 'r', imp.PY_SOURCE))
    fp.close()

    # rebuild the widget
    # - find and hide the existing widget
    # - create a new widget in the existing parent
    parent = slicer.util.findChildren(name='%s Reload' % moduleName)[0].parent()
    for child in parent.children():
      try:
        child.hide()
      except AttributeError:
        pass
    # Remove spacer items
    item = parent.layout().itemAt(0)
    while item:
      parent.layout().removeItem(item)
      item = parent.layout().itemAt(0)
    # create new widget inside existing parent
    globals()[widgetName.lower()] = eval(
        'globals()["%s"].%s(parent)' % (moduleName, widgetName))
    globals()[widgetName.lower()].setup()

  def onDownloadMRIData(self):
   tester = CardiacMRIToolkitSelfTestCase()
   tester.setUp()
   tester.test_DownloadMRIData()
   
  def onLAEndoSegmentation(self):
   tester = CardiacMRIToolkitSelfTestCase()
   tester.test_LAEndoSegmentation()

  def onAxialDilate(self):
   tester = CardiacMRIToolkitSelfTestCase()
   tester.test_AxialDilate()

  def onBooleanRemove(self):
   tester = CardiacMRIToolkitSelfTestCase()
   tester.test_BooleanRemove()

  def onDownloadLAWallCleanup(self):
   tester = CardiacMRIToolkitSelfTestCase()
   tester.test_DownloadLAWallCleanup()

  def onPVAntrumCut(self):
   tester = CardiacMRIToolkitSelfTestCase()
   tester.test_PVAntrumCut()

  def onEndoNVIsosurface(self):
   tester = CardiacMRIToolkitSelfTestCase()
   tester.test_EndoNVIsosurface()

  def onAutomaticLeftAtrialScar(self):
   tester = CardiacMRIToolkitSelfTestCase()
   tester.test_AutomaticLeftAtrialScar()

#
# CardiacMRIToolkitSelfTestLogic
#

class CardiacMRIToolkitSelfTestLogic:
  """This class should implement all the actual
  computation done by your module.  The interface
  should be such that other python code can import
  this class and make use of the functionality without
  requiring an instance of the Widget
  """
  def __init__(self):
    pass

  def hasImageData(self,volumeNode):
    """This is a dummy logic method that
    returns true if the passed in volume
    node has valid image data
    """
    if not volumeNode:
      print('no volume node')
      return False
    if volumeNode.GetImageData() == None:
      print('no image data')
      return False
    return True

class CardiacMRIToolkitSelfTestCase(unittest.TestCase):
  """
  This is the test case for your scripted module.
  """

  def delayDisplay(self,message,msec=1000):
    """This utility method displays a small dialog and waits.
    This does two things: 1) it lets the event loop catch up
    to the state of the test so that rendering and widget updates
    have all taken place before the test continues and 2) it
    shows the user/developer/tester the state of the test
    so that we'll know when it breaks.
    """
    print(message)
    self.info = qt.QDialog()
    self.infoLayout = qt.QVBoxLayout()
    self.info.setLayout(self.infoLayout)
    self.label = qt.QLabel(message,self.info)
    self.infoLayout.addWidget(self.label)
    qt.QTimer.singleShot(msec, self.info.close)
    self.info.exec_()

  def setUp(self):
    """ Do whatever is needed to reset the state - typically a scene clear will be enough.
    """
    self.delayDisplay("Closing the scene")
    layoutManager = slicer.app.layoutManager()
    layoutManager.setLayout(slicer.vtkMRMLLayoutNode.SlicerLayoutConventionalView)
    slicer.mrmlScene.Clear(0)

  def clickAndDrag(self,widget,button='Left',start=(10,10),end=(10,40),steps=20,modifiers=[]):
    """ Send synthetic mouse events to the specified widget (qMRMLSliceWidget or qMRMLThreeDView)
    button : "Left", "Middle", "Right", or "None"
    start, end : window coordinates for action
    steps : number of steps to move in
    modifiers : list containing zero or more of "Shift" or "Control"
    """
    style = widget.interactorStyle()
    interactor = style.GetInteractor()
    if button == 'Left':
      down = style.OnLeftButtonDown
      up = style.OnLeftButtonUp
    elif button == 'Right':
      down = style.OnRightButtonDown
      up = style.OnRightButtonUp
    elif button == 'Middle':
      down = style.OnMiddleButtonDown
      up = style.OnMiddleButtonUp
    elif button == 'None' or not button:
      down = lambda : None
      up = lambda : None
    else:
      raise Exception("Bad button - should be Left or Right, not %s" % button)
    if 'Shift' in modifiers:
      interactor.SetShiftKey(1)
    if 'Control' in modifiers:
      interactor.SetControlKey(1)
    interactor.SetEventPosition(*start)
    down()
    for step in xrange(steps):
      frac = float(step)/steps
      x = int(start[0] + frac*(end[0]-start[0]))
      y = int(start[1] + frac*(end[1]-start[1]))
      interactor.SetEventPosition(x,y)
      style.OnMouseMove()
    up()
    interactor.SetShiftKey(0)
    interactor.SetControlKey(0)

  def runTest(self):
    """Run as few or as many tests as needed here.
    """
    self.setUp()
    self.test_DownloadMRIData()
    self.test_LAEndoSegmentation()
    self.test_AxialDilate()
    self.test_BooleanRemove()
    self.test_DownloadLAWallCleanup()
    self.test_PVAntrumCut()
    self.test_EndoNVIsosurface()
    self.test_AutomaticLeftAtrialScar()
    
  def test_DownloadMRIData(self):
    self.delayDisplay("Starting the test")
    #
    # first, get some data
    #
    
    #import urllib
    #downloads = (
    #    ('http://slicer.kitware.com/midas3/download/item/100513/LGE-MRI.nrrd', 'LGE-MRI.nrrd', slicer.util.loadVolume),
    #    )
    vl = slicer.modules.volumes.logic() 
    global inputVolume
    inputVolume = vl.AddArchetypeVolume('http://www.slicer.org/slicerWiki/images/b/b2/LGE-MRI.nrrd', 'LGE-MRI', 0)
    #if vol != None:
    #  appLogic = slicer.app.applicationLogic()
    #  selectionNode = appLogic.GetSelectionNode()
    #  selectionNode.SetReferenceActiveVolumeID(vol)
    #  appLogic.PropagateVolumeSelection()
    
    #for url,name,loader in downloads:
    #  filePath = slicer.app.temporaryPath + '/' + name
    #  if not os.path.exists(filePath) or os.stat(filePath).st_size == 0:
    #    print('Requesting download %s from %s...\n' % (name, url))
    #    urllib.urlretrieve(url, filePath)
    #  if loader:
    #    print('Loading %s...\n' % (name,))
    #    loader(filePath)
    self.delayDisplay('Finished with LGE-MRI Image download and loading\n')
    self.delayDisplay('Test passed!')
    #try:
    #  logic = CardiacMRIToolkitSelfTestLogic()
    #  mainWindow = slicer.util.mainWindow()
    #  layoutManager = slicer.app.layoutManager()
    #  threeDView = layoutManager.threeDWidget(0).threeDView()

    #  redWidget = layoutManager.sliceWidget('Red')
    #  redController = redWidget.sliceController()
    #  greenWidget = layoutManager.sliceWidget('Green')
    #  greenController = greenWidget.sliceController()

    #  self.delayDisplay('LGE-MRI Image')
      #mainWindow.moduleSelector().selectModule('Models')
    #  redWidget.sliceController().setSliceVisible(True);
    #  greenWidget.sliceController().setSliceVisible(True);

      #self.delayDisplay('Rotate')
      #self.clickAndDrag(threeDView)

      #self.delayDisplay('Zoom')
      #threeDView = layoutManager.threeDWidget(0).threeDView()
      #self.clickAndDrag(threeDView,button='Right')

    #  self.delayDisplay('Scroll Slices')
    #  for offset in xrange(-20,20,2):
    #    redController.setSliceOffsetValue(offset)

    #  self.delayDisplay('Test passed!')
      
    #except Exception, e:
    #  import traceback
    #  traceback.print_exc()
    #  self.delayDisplay('Test caused exception!\n' + str(e))

  def test_LAEndoSegmentation(self):
    self.delayDisplay("Starting the test")
    #
    # Get the endo segmentation data
    #
    import urllib
    downloads = (
       ('http://slicer.kitware.com/midas3/download/item/100514/LA+Endo.nrrd', 'LA_Endo.nrrd', slicer.util.loadVolume),
       )
        
    vl = slicer.modules.volumes.logic() 
    global endoSegVolume
    endoSegVolume = vl.AddArchetypeVolume('http://slicer.kitware.com/midas3/download/item/100514/LA+Endo.nrrd', 'LA_Endo', 0)
    
   # for url,name,loader in downloads:
   #   filePath = slicer.app.temporaryPath + '/' + name
   #   if not os.path.exists(filePath) or os.stat(filePath).st_size == 0:
   #     print('Requesting download %s from %s...\n' % (name, url))
   #     urllib.urlretrieve(url, filePath)
   #   if loader:
   #     print('Loading %s...\n' % (name,))
   #     loader(filePath)
    
    self.delayDisplay('Finished with endocardium segmentation download and loading\n')
         
    volumeNode = slicer.util.getNode(pattern="LA_Endo")
    
    m = slicer.util.mainWindow()
    m.moduleSelector().selectModule('Volumes')
    self.delayDisplay("Entered Volumes module")

    selector = slicer.util.findChildren(name='ActiveVolumeNodeSelector')[0]
    checkBox = slicer.util.findChildren(name='LabelMapCheckBox')[0]
    
    selector.setCurrentNode(volumeNode)
    checkBox.checked = True
    self.delayDisplay("Made the endocardium segmentation into a label map")
    self.delayDisplay('Endocardium segmentation test passed!')
    #endo_label = vl.SetVolumeAsLabelMap(endo_vol, 0);
    
    #try:
    #  logic = CardiacMRIToolkitSelfTestLogic()
    #  mainWindow = slicer.util.mainWindow()
    #  layoutManager = slicer.app.layoutManager()
    #  threeDView = layoutManager.threeDWidget(0).threeDView()
      
    #  redWidget = layoutManager.sliceWidget('Red')
    #  redController = redWidget.sliceController()
    #  greenWidget = layoutManager.sliceWidget('Green')
    #  greenController = greenWidget.sliceController()

    #  self.delayDisplay('Left Atrium Endocardial Segmentation')
      #mainWindow.moduleSelector().selectModule('Models')
    #  redWidget.sliceController().setSliceVisible(True);

    #  self.delayDisplay('Rotate')
    #  self.clickAndDrag(threeDView)

    #  self.delayDisplay('Zoom')
    #  threeDView = layoutManager.threeDWidget(0).threeDView()
    #  self.clickAndDrag(threeDView,button='Right')

    #  self.delayDisplay('Scroll Slices')
    #  for offset in xrange(-20,20,2):
    #    redController.setSliceOffsetValue(offset)

     # self.delayDisplay('Test passed!')
      
    #except Exception, e:
    #  import traceback
    #  traceback.print_exc()
    #  self.delayDisplay('Test caused exception!\n' + str(e))
    
    global endoSegID
    endoSegID = endoSegVolume.GetID()
    
  def test_AxialDilate(self):
    self.delayDisplay("Starting the test")
    #
    # Create the label image to store the output
    #
    self.delayDisplay( 'Creating output image to store result' )
    volumesLogic = slicer.modules.volumes.logic()
    global epiOutputVolume 
    epiOutputVolume = volumesLogic.CreateAndAddLabelVolume( slicer.mrmlScene, endoSegVolume, 'Epi Segmentation Output' )
    self.delayDisplay( 'Running Axial Dilate' )
    
    if epiOutputVolume != None and endoSegVolume != None:
      param = {}
      param['targetFileName'] = endoSegVolume.GetID()
      param['outputFileName'] = epiOutputVolume.GetID()
      slicer.cli.run( slicer.modules.cmrtoolkitaxialdilate, None, param, wait_for_completion=True )
      self.delayDisplay('Axial Dilate test passed!')
    else:
      self.delayDisplay('Test Failed!')    
    
    #volumesLogic.SaveArchetypeVolume( 'epiOutputVolume.nrrd', epiOutputVolume )
    
  def test_BooleanRemove(self):
    self.delayDisplay( "Creating output image to store result" )
    volumesLogic = slicer.modules.volumes.logic()
    global wallOutputVolume 
    wallOutputVolume = volumesLogic.CreateAndAddLabelVolume( slicer.mrmlScene, epiOutputVolume, 'Wall Segmentation Output' )
    
    endoSegVol = slicer.mrmlScene.GetNodeByID(endoSegID)
    
    self.delayDisplay( 'Running Boolean Remove' )
    if epiOutputVolume != None and endoSegVolume != None :
      param = {}
      param['inputVolume1'] = epiOutputVolume.GetID()
      param['inputVolume2'] = endoSegVol.GetID()
      param['outputVolume'] = wallOutputVolume.GetID()
      slicer.cli.run( slicer.modules.cmrtoolkitbooleanremovefilter, None, param, wait_for_completion=True )
      self.delayDisplay('Boolean Remove test passed!')
    else:
      self.delayDisplay('Test Failed!')    
      
  def test_DownloadLAWallCleanup(self):
    self.delayDisplay("Starting the test")
    #
    # Download the data
    #
    #downloads = (
    #   ('http://slicer.kitware.com/midas3/download/item/100534/LA+Wall.nrrd', 'LA_Wall.nrrd', slicer.util.loadVolume),
    #   )
        
    vl = slicer.modules.volumes.logic() 
    global wallSegVolume
    wallSegVolume = vl.AddArchetypeVolume('http://slicer.kitware.com/midas3/download/item/100534/LA+Wall.nrrd', 'LA_Wall', 0)
    
    self.delayDisplay('Finished with wall segmentation download and loading\n')
         
    volumeNode = slicer.util.getNode(pattern="LA_Wall")
    
    m = slicer.util.mainWindow()
    m.moduleSelector().selectModule('Volumes')
    self.delayDisplay("Entered Volumes module")

    selector = slicer.util.findChildren(name='ActiveVolumeNodeSelector')[0]
    checkBox = slicer.util.findChildren(name='LabelMapCheckBox')[0]
    
    selector.setCurrentNode(volumeNode)
    checkBox.checked = True
    self.delayDisplay("Made the wall segmentation into a label map")
    self.delayDisplay('LA wall cleanup test passed!')

  def test_PVAntrumCut(self):
    self.delayDisplay("Starting the test")  
      
    volumesLogic = slicer.modules.volumes.logic()
    global antrumCutOutputVolume 
    antrumCutOutputVolume = volumesLogic.CreateAndAddLabelVolume( slicer.mrmlScene, endoSegVolume, 'Endocardium No Veins Segmentation Output' )
    
    # Load wall and endo segmentation volumes from previous steps
     
    if endoSegVolume != None and wallSegVolume != None:
      param = {}
      param['endoLayer'] = endoSegVolume.GetID()
      param['wallLayer'] = wallSegVolume.GetID()
      param['endoNoVeins'] = antrumCutOutputVolume.GetID()
      slicer.cli.run( slicer.modules.cmrtoolkitpvantrumcut, None, param, wait_for_completion=True )
      self.delayDisplay("PV Antrum Cut test passed!")
    else:
      self.delayDisplay('Test Failed!')   
    
  def test_EndoNVIsosurface(self):
    self.delayDisplay("Starting the test")
    # Initialize the parameters to run the Model Maker Module   
     
    if antrumCutOutputVolume != None :
      param = {}
      param['InputVolume'] = antrumCutOutputVolume.GetID()
      
      # Make a new model hierarchy node if needed
      numNodes = slicer.mrmlScene.GetNumberOfNodesByClass( "vtkMRMLModelHierarchyNode" )
      global outHierarchy 
      outHierarchy = None
      for n in xrange(numNodes):
        node = slicer.mrmlScene.GetNthNodeByClass( n, "vtkMRMLModelHierarchyNode" )
        if node.GetName() == "Endo Model":
          outHierarchy = node
          break
        
      outHierarchy = slicer.vtkMRMLModelHierarchyNode()
      outHierarchy.SetScene( slicer.mrmlScene )
      outHierarchy.SetName( "Endo Model" )
      slicer.mrmlScene.AddNode( outHierarchy )
      
      self.delayDisplay("Starting Model Maker Module")
      param['ModelSceneFile'] = outHierarchy
      slicer.cli.run( slicer.modules.modelmaker, None, param, wait_for_completion=True )
      self.delayDisplay('Endocardium isosurface test passed!')
    else:
      self.delayDisplay('Test Failed!') 
    
  def test_AutomaticLeftAtrialScar(self):
    # Run Auto Scar Module
    self.delayDisplay("Starting the test")
    
    volumesLogic = slicer.modules.volumes.logic()
    global scarOutputVolume 
    scarOutputVolume = volumesLogic.CreateAndAddLabelVolume( slicer.mrmlScene, wallSegVolume, 'Scar Segmentation Output' )
    
    if inputVolume != None and wallSegVolume != None and antrumCutOutputVolume != None and scarOutputVolume != None:
      param = {}
      param['lgefn'] = inputVolume.GetID()
      param['lawallfn'] = wallSegVolume.GetID()
      param['laendofn'] = antrumCutOutputVolume.GetID()
      param['outputfn'] = scarOutputVolume.GetID()
      slicer.cli.run( slicer.modules.carmaautomaticleftatrialscar, None, param, wait_for_completion=True )
      self.delayDisplay('Automatic left atrial scar test passed!')
    else:
      self.delayDisplay('Automatic left atrial scar test failed!')
  
    self.delayDisplay("Starting the scar isosurface test")
    if scarOutputVolume != None:
      # Initialize the input parameters to run the Model Maker Module
      param = {}
      param['InputVolume'] = scarOutputVolume.GetID()
      
      # Make a new model hierarchy node if needed
      numNodes = slicer.mrmlScene.GetNumberOfNodesByClass( "vtkMRMLModelHierarchyNode" )
      outHierarchy = None
      for n in xrange(numNodes):
        node = slicer.mrmlScene.GetNthNodeByClass( n, "vtkMRMLModelHierarchyNode" )
        if node.GetName() == "Scar Model":
          outHierarchy = node
          break
        
      outHierarchy = slicer.vtkMRMLModelHierarchyNode()
      outHierarchy.SetScene( slicer.mrmlScene )
      outHierarchy.SetName( "Scar Model" )
      slicer.mrmlScene.AddNode( outHierarchy )
      
      param['ModelSceneFile'] = outHierarchy
      slicer.cli.run( slicer.modules.modelmaker, None, param, wait_for_completion=True )
      self.delayDisplay('Scar isosurface test passed!')
    else:
      self.delayDisplay('Scar isosurface making test failed!')
    
  
    



